@@ -1,9 +1,9 @@
 package Test::Smoke::Util;
 use strict;
 
-# $Id: Util.pm 906 2005-09-09 16:26:12Z abeltje $
+# $Id: Util.pm 968 2006-05-25 19:13:03Z abeltje $
 use vars qw( $VERSION @EXPORT @EXPORT_OK );
-$VERSION = '0.45';
+$VERSION = '0.47';
 
 use base 'Exporter';
 @EXPORT = qw( 
@@ -447,7 +447,7 @@ sub grepccmsg {
         'bcc32' => # BORLAND 5.5 on MSWin32
             # Warning Wnnn filename line: warning description
             # Error Ennn:: error description
-            '(^(?:(?:Warning W\d+ .+? \d+)|(?:Error E\d+)): .+?$)',
+            '(^(?:(?:Warning W)|(?:Error E))\d+ .+? \d+: .+?$)',
     );
     exists $OS2PAT{ lc $cc } or $cc = 'gcc';
     my $pat = $OS2PAT{ lc $cc };
@@ -466,7 +466,7 @@ sub grepccmsg {
         $error{ "Couldn't examine '$logfile' for compiler warnings." } = 1;
     }
 
-    $error{ $1 } ||= $indx++ while $smokelog =~ /$pat/smg;
+    $error{ $1 } ||= $indx++ while $smokelog =~ /$pat/mg;
 
     # I need to think about this IRIX/$Config{cc} thing
 #    if ($cc eq 'irix') {
@@ -502,12 +502,19 @@ sub get_local_patches {
         return @lpatches;
     }
     $verbose and print " open ok\n";
-    my $seen;
+    my( $seen, $patchnum );
     while ( <PLEVEL> ) {
+        $patchnum = $1 if /#define PERL_PATCHNUM\s+(\d+)/;
         $seen && /^\s*,"(.+)"/ and push @lpatches, $1;
         /^\s*static.+?local_patches\[\]/ and $seen++;
     }
     close PLEVEL;
+    if ( defined $patchnum ) {
+        @lpatches = map {
+            s/^(MAINT|DEVEL)$/$1$patchnum/;
+            $_;
+        } @lpatches;
+    }
     $verbose and do { local $"=';'; print "Patches: '@lpatches'\n" };
     return @lpatches;
 }
@@ -680,16 +687,18 @@ sub get_patch {
     local *PATCHLEVEL_H;
     my $patchlevel_h = File::Spec->catfile( $ddir, 'patchlevel.h' );
     if ( open PATCHLEVEL_H, "< $patchlevel_h" ) {
-        my $declaration_seen = 0;
+        my( $declaration_seen, $patchnum ) = ( 0, 0 );
         while ( <PATCHLEVEL_H> ) {
+            $patchnum = $1 if /#define PERL_PATCHNUM\s+(\d+)/;
             $declaration_seen ||= /local_patches\[\]/;
-            $declaration_seen && /^\s+,"(?:DEVEL|MAINT)(\d+)|(RC\d+)"/ or next;
-            $patch_level = $1 || $2 || '?????';
+            $declaration_seen &&
+                /^\s+,"(?:(?:DEVEL|MAINT)(\d+)?)|(RC\d+)"/ or next;
+            $patch_level = $patchnum || $1 || $2 || '?????';
             if ( $patch_level =~ /^RC/ ) {
                 $patch_level = version_from_patchlevel_h( $ddir ) .
                                "-$patch_level";
             } else {
-                $patch_level .= '(+)';
+                $patch_level .= $patchnum ? "" : '(+)';
             }
         }
     }
@@ -1152,7 +1161,7 @@ sub skip_filter {
     m,^(   )?### , ||
     # Clean up Win32's output
     m,^(?:\.\.[/\\])?[\w/\\-]+\.*ok$, ||
-    m,^(?:\.\.[/\\])?[\w/\\-]+\.*ok\s+\d+(\.\d+)?s$, ||
+    m,^(?:\.\.[/\\])?[\w/\\-]+\.*ok\s+\d+(\.\d+)?\s*m?s$, ||
     m,^(?:\.\.[/\\])?[\w/\\-]+\.*ok\,\s+\d+/\d+\s+skipped:, ||
     m,^(?:\.\.[/\\])?[\w/\\-]+\.*skipped[: ], ||
     m,^\t?x?copy , ||
